@@ -1,6 +1,6 @@
 package JE::Object::Number;
 
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 
 
 use strict;
@@ -12,12 +12,14 @@ our @ISA = 'JE::Object';
 
 use Scalar::Util 'blessed';
 
+require JE::Code;
 require JE::Number;
 require JE::Object;
 require JE::Object::Function;
 require JE::String;
 
-
+import JE::Code 'add_line_number';
+sub add_line_number;
 
 =head1 NAME
 
@@ -171,7 +173,7 @@ sub _new_constructor {
 			function => sub {
 				my $self = shift;
 				die JE::Object::Error::TypeError->new(
-					$global,
+					$global, add_line_number
 					"Argument to " .
 					"Number.prototype.toString is not"
 					. " a " .
@@ -224,7 +226,7 @@ sub _new_constructor {
 			function => sub {
 				my $self = shift;
 				die JE::Object::Error::TypeError->new(
-					$global,
+					$global, add_line_number
 					"Argument to " .
 					"Number.prototype.toLocaleString ".
 					"is not"
@@ -250,7 +252,7 @@ sub _new_constructor {
 			function => sub {
 				my $self = shift;
 				die JE::Object::Error::TypeError->new(
-					$global,
+					$global, add_line_number
 					"Argument to " .
 					"Number.prototype.valueOf is not"
 					. " a " .
@@ -274,7 +276,7 @@ sub _new_constructor {
 			function => sub {
 my $self = shift;
 die JE::Object::Error::TypeError->new(
-	$global,
+	$global, add_line_number
 	"Argument to " .
 	"Number.prototype.toFixed is not"
 	. " a " .
@@ -317,7 +319,7 @@ return JE::String->new($global, sprintf "%.${places}f", $num);
 			function => sub {
 my $self = shift;
 die JE::Object::Error::TypeError->new(
-	$global,
+	$global, add_line_number
 	"Argument to " .
 	"Number.prototype. toExponential is not"
 	. " a " .
@@ -361,7 +363,7 @@ return JE::String->new($global, $result);
 			function => sub {
 my $self = shift;
 die JE::Object::Error::TypeError->new(
-	$global,
+	$global, add_line_number
 	"Argument to " .
 	"Number.prototype. toPrecision is not"
 	. " a " .
